#ifndef PPSSUMM_BASEREAL
#define PPSSUMM_BASEREAL

#include <string>
#include <cstdlib>
#include <iostream>

template <typename T>
class BaseReal 
{
    public:
    BaseReal() : _base_real(0.0) { }
    BaseReal(const T& s ) : _base_real(s) { }
    BaseReal( const BaseReal& s ) : _base_real(s._base_real) { }

    // Accessor
    T get() const { return _base_real; }
    T set( const T& s ) 
    { 
        T tmp = _base_real;
        _base_real = s;
        return tmp; 
    }

    // Operators
    bool operator<(const BaseReal& s) const
    { 
        return _base_real < s._base_real;
    }
    bool operator>(const BaseReal& s) const
    { 
        return _base_real > s._base_real;
    }
    bool operator==(const BaseReal& s) const
    { 
        return _base_real == s._base_real;
    }

    // prefix and postfix 
    BaseReal operator=(const T& s)
    { 
        _base_real = s;
        return *this;
    }

    BaseReal operator=(const BaseReal& s)
    { 
        _base_real = s.get();
        return *this;
    }

    //
    BaseReal operator++()
    { 
	BaseReal tmp(*this);
        ++_base_real;
        return tmp;
    }
    BaseReal operator++(int i)
    { 
        ++_base_real;
        return *this;
    }

    BaseReal operator--()
    { 
	BaseReal tmp(*this);
        --_base_real;
        return tmp;
    }
    BaseReal operator--(int i)
    { 
        --_base_real;
        return *this;
    }

    BaseReal operator+=(const T& i)
    { 
        _base_real+=i;
        return *this;
    }
    BaseReal operator-=(const T& i)
    { 
        _base_real-=i;
        return *this;
    }

    friend std::ostream &operator<<(std::ostream& os , const BaseReal& s)
    {
        os << s.get();
        return os;
    }
    private:
    T _base_real;
};
#endif
