#ifndef PPSSUMM_DATAMODE
#define PPSSUMM_DATAMODE
#include <string>
#include <sstream>
#include <vector>
#include <algorithm>

#include "ppssumm_typedef.h"
#include "string_algo.h"
#include "Tracer.h"
#include "errstr.h"


class Datamode 
{
    public:
    Datamode() : _datamode() { }
    Datamode( const char* s ) : _datamode() {  _datamode.push_back(s); }
    Datamode( const std::string& s ) : _datamode() { _datamode.push_back(s); }

    Datamode( const Datamode& s ) : _datamode() 
    {
	stringlist::const_iterator SIB = s.begin();
	stringlist::const_iterator SIE = s.end();
	for( ; SIB != SIE ; SIB++ )
	{
	    _datamode.push_back( std::string( *SIB ) );
	}

	}
	Datamode( const stringlist& s ) : _datamode() 
	{
	stringlist::const_iterator SIB = s.begin();
	stringlist::const_iterator SIE = s.end();
	for( ; SIB != SIE ; SIB++ )
	{
	    _datamode.push_back( std::string( *SIB ) );
	}
    }

    stringlist::const_iterator begin() const { return _datamode.begin(); }
    stringlist::iterator begin() { return _datamode.begin(); }
    stringlist::const_iterator end() const { return _datamode.end(); }
    stringlist::iterator end()  { return _datamode.end(); }

    /*
    stringlist::const_iterator begin() const { return _datamode.begin(); }
    stringlist::iterator begin() { return _datamode.begin(); }

    stringlist::const_iterator end() const { return _datamode.end(); }
    stringlist::iterator end() { return _datamode.end(); }
    */

    void push_back(const std::string& s)
    {
	_datamode.push_back(s);
    }

    std::string asString() const
    {
	bool sep = false;
	sbmap seen;
	std::stringstream ss;
	ss << "";
	for( stringlist::const_iterator i = _datamode.begin() 
	    ; i != _datamode.end() 
	    ; i++ 
	) {   
	    std::string r = std::string(*i);
	    std::transform( r.begin()
		, r.end()
		, r.begin()
		, ppssumm_internal::toUpper
	    );

	    if( seen.find(r) == seen.end() )
	    {
		seen[r] = true;
		std::transform( r.begin()+1
		    , r.end()
		    , r.begin()+1
		    , ppssumm_internal::toLower
		);
		if (sep) ss << ", ";
		ss << r;
		sep = true;
	    }
	}
	return ss.str();
    }

    friend std::ostream &operator<<(std::ostream& os , const Datamode& s)
    {
	std::string r = s.asString();
	if (r.empty())
	{
	    os << "&nbsp;";
	}
	else
	{
	    os << r;
	}
	return os;
    }

    private:
    stringlist _datamode;
};

class MakeDatamode
{
    public:
    Datamode operator()() { return Datamode(); };
    Datamode operator()(const Datamode& s) { return Datamode( s ); };
    Datamode operator()(const char* s) { return Datamode(s); };
    Datamode operator()(const std::string& s) { return Datamode(s); };
};
#endif
