#ifndef PPSSUMM_EPIC_SUMMARY
#define PPSSUMM_EPIC_SUMMARY

#include <vector>
#include <string>
#include <iostream>
#include <algorithm>
#include <iterator>
#include "ppssumm_typedef.h"
#include "PpssummCommon.h"
#include "Pindex.h"
#include "Eindex.h"
#include "ppssumm_algo.h"
#include "string_algo.h"
#include "key_algo.h"
#include "STime.h"
#include "Dal.h"
#include "param.h"
#include "Tracer.h"
#include "errstr.h"

class EpicSummary 
{
    public:
    EpicSummary(  DataSet* fitsum , Pindex& pindex ) :
	_epicmultiexposureproduct(pindex) 
	,_epicexposuresprocessed(fitsum,pindex)
	,_epicexposurespecificproduct(pindex)
	,_epicsourcesprocessed(pindex)
	,_epicsourcespecificproduct(pindex)
    { }

    friend std::ostream &operator<<(std::ostream& os , const EpicSummary& es) {

	// Page menu
	os  << "<div id=\"pagemenu\">\n"
	    << "<ul>\n"
	    << "<li><a href=\"#mulitexposureproduct\">Multi-Exposure Products</a></li>\n"
	    << "<li><a href=\"#exposuresprocessed\">Exposures processed by PPS</a></li>\n"
	    << "<li><a href=\"#sourcesprocessed\">Sources processed by PPS</a></li>\n"
	    << "</ul>\n"
	    << "<br clear=\"left\" />"
	    << "<hr />\n"
	    << "</div>"
	;
	// Now the fun begins

	os  << "<div id=\"normaltable\">"  << "\n"
	    << "<a name=\"mulitexposureproduct\">&nbsp;</a>"
	    << "<h3>Multi-Exposure Products</h3>"
	;
	if (es._epicmultiexposureproduct.size())
	{
	    os	<< "<table class=\"epicmultiexposureproduct_table\">"
		<< es._epicmultiexposureproduct
		<< "</table>"
	    ;
	}
	else
	{
	    os << "<p class=\"noprod\">No multi-exposure products were produced.</p>";
	}
	os << "<hr />";
	os << "</div>";

	// EXPOSURES

	os << "<div id=\"exposuresprocessed\">" << "\n";
	os << "<a name=\"exposuresprocessed\">&nbsp;</a>" << "\n";
	os << "<h3>Exposures processed by PPS</h3>" << "\n";
	if (es._epicexposuresprocessed.size())
	{
	    // Exposures processed table
	    os << "<div id=\"widetable\">" << "\n";
	    os << "<table class=\"epicexposureprocessed_table\">" << "\n";
	    os << es._epicexposuresprocessed;
	    os << "</table>" << "\n";
	    os << "<hr />" << "\n";
	    os << "</div>" << "\n";

	    // Exposure specific products
	    os << "<div id=\"exposurespecificproducts\">" << "\n";
	    os << "<h3>Exposure Specific Products</h3>" << "\n";
	    if( es._epicexposurespecificproduct.size() )
	    {
		for ( Epicexposuresprocessed::const_iterator CI = es._epicexposuresprocessed.begin()
		    ; CI != es._epicexposuresprocessed.end()
		    ; ++CI
		){

		    // Select exposurespecificproduct for this instrument and exposure.
		    Epicexposurespecificproduct tmp;
		    for( Epicexposurespecificproduct::const_iterator CI2 = es._epicexposurespecificproduct.begin()
			; CI2 != es._epicexposurespecificproduct.end()
			; ++CI2
		    ){
			if( CI2->select( CI->instrument() , CI->exp_id() ) ) tmp.push_back( *CI2 );
		    }

		    // Repeat exposure row ahead of exposure specific product table
		    os << "<div id=\"widetable\">" << "\n";
		    os << "<a name=\"" <<  ppssumm_internal::makekey( CI->instrument() , CI->exp_id() ) << "\">&nbsp;</a>" << "\n";
		    os << "<table class=\"epicexposurespecific_toptable\">" << "\n";
		    es._epicexposuresprocessed.title(os);
		    os << *CI;
		    os << "</table>" << "\n";
		    os << "</div>" << "\n";

		    // Write out exposurespecificproduct table
		    if (tmp.size())
		    {
			std::sort(tmp.begin(), tmp.end() , std::less<EpicexposurespecificproductRow>() );
		    os << "<div id=\"normaltable\">" << "\n";
			os << "<table class=\"epicexposurespecific_toptable\">" << "\n";
			os << tmp;
			os << "</table>" << "\n";
		    os << "</div>" << "\n";
		    }
		    else
		    {
			os << "<p class=\"noprod\">No exposure specific products were produced.</p>" << "\n";
		    }
		}
	    }
	    else
	    {
		os << "<p class=\"noprod\">No exposure specific products were produced.</p>" << "\n";
	    }
	    os << "<hr />" << "\n";
	    os << "</div>" << "\n";
	}
	else
	{
	    os << "<p class=\"noprod\">No EPIC exposures processed by the PPS.</p>" << "\n";
	    os << "<hr />" << "\n";
	}
	os << "</div>" << "\n";


	// SOURCES

	os << "<div class=\"epicsourcesprocessed\">" << "\n";
	os << "<a name=\"sourcesprocessed\">&nbsp;</a>" << "\n";
	os << "<h3>Sources processed by PPS</h3>" << "\n";
	if (es._epicsourcesprocessed.size())
	{
	    // Sources processed table
	    os << "<div id=\"widetable\">" << "\n";
	    os << "<table class=\"epicsourceprocessed_table\">" << "\n";
	    os << es._epicsourcesprocessed;
	    os << "</table>" << "\n";
	    os << "<hr />" << "\n";
	    os << "</div>" << "\n";

	    // Source specific products
	    os << "<div class=\"epicsourcespecificproduct\">" << "\n";
	    os << "<h3>Source Specific Products</h3>" << "\n";
	    if( es._epicsourcespecificproduct.size() )
	    {	
		for ( Epicsourcesprocessed::const_iterator CI = es._epicsourcesprocessed.begin()
		    ; CI != es._epicsourcesprocessed.end()
		    ; ++CI
		){
		    // Select source specific product for this instrument and source.
		    Epicsourcespecificproduct tmp;
		    for( Epicsourcespecificproduct::const_iterator CI2 = es._epicsourcespecificproduct.begin()
			; CI2 != es._epicsourcespecificproduct.end()
			; ++CI2
		    ){

			if( CI2->select( CI->instrument() , CI->exp_id(), CI->src_num() ) ) 
			{
			    tmp.push_back( *CI2 );
			}
			else
			{
			}
		    }

		    // Repeat source row ahead of source specific product table
		    std::string key = ppssumm_internal::makekey(CI->instrument() , CI->exp_id() ,CI->src_num());
		    os  << "<div id=\"widetable\">" << "\n"
			<< "<a name=\"" << key << "\">&nbsp;</a>" << "\n"
			<< "<table class=\"epicsourcespecific_toptable\">" << "\n"
		    ;
		    es._epicsourcesprocessed.title(os);
		    os	<< *CI
			<< "</table>" << "\n"
			<< "</div>" << "\n"
		    ;

		    // Write out sourcespecificproduct table
		    if (tmp.size())
		    {
			std::sort(tmp.begin(), tmp.end() , std::less<EpicsourcespecificproductRow>() );
			os  << "<div id=\"normaltable\">" << "\n"
			    << "<table class=\"epicsourcespecific_toptable\">" << "\n"
			    << tmp
			    << "</table>" << "\n"
			    << "</div>"
			;
		    }
		    else
		    {
			os << "<p class=\"noprod\">No source specific products were produced.</p>" << "\n"
			;
		    }
		}
	    }
	    else
	    {
		os  << "<p class=\"noprod\">No source specific products were produced.</p>" << "\n"
		;
	    }
	    os  << "<hr />" << "\n"
		<< "</div>" << "\n"
	    ;
	}
	else
	{
	    os  << "<p class=\"noprod\">No EPIC sources processed by the PPS.</p>" << "\n"
	    ;
	    os  << "<hr />" << "\n";
	}
	os  << "</div>" << "\n";

        return os;
    }
   
    const Epicexposuresprocessed* epicexposuresprocessed() const { return &_epicexposuresprocessed; }
 
    private:
    Epicmultiexposureproduct _epicmultiexposureproduct;
    Epicexposuresprocessed _epicexposuresprocessed;
    Epicexposurespecificproduct _epicexposurespecificproduct;
    Epicsourcesprocessed _epicsourcesprocessed;
    Epicsourcespecificproduct _epicsourcespecificproduct;

};

#endif
