#ifndef PPSSUMM_HTMLTAG
#define PPSSUMM_HTMLTAG

#include <iostream>
#include <sstream>
#include <string>

/* The contents between the tags are stored in this*/
using namespace std;

class HtmlTag
: public string
{
    public:
    HtmlTag()
    : string("") , tag_("") , extra_("") , pre_("") , post_("") 
    {  }

    HtmlTag(string t, string e="" , string c="" , string pre="" , string post="")
    : string(c) , tag_(t) , extra_(e) , pre_(pre) , post_(post) 
    {  }

    HtmlTag(string t, string e , HtmlTag ht , string pre="" , string post="" ) 
    : string( ht.print() ) , tag_(t) , extra_(e) , pre_(pre) , post_(post) 
    { }
    /* The above be:
     *    c  content between tags (empty < /> generated if empty)
     *    t  the tag <t></t>
     *    e  tag extra <t e></t> |<t e/>
     *    pre  pretag insert  pre<t e></t> | pre<t e/>
     *    post  posttag insert  <t e></t>post | <t e/>post
     *
     *     Why ?
     *    post for example is good for putting a '\n' 
     *    outside of <pre></pre> tags
     */
    
    HtmlTag(HtmlTag ht , string t, string e ,  string pre="" , string post="" ) 
    : string( ht.begin() , ht.end() ) , tag_(t) , extra_(e) , pre_(pre) , post_(post) 
    { }


    HtmlTag(const HtmlTag& ht)
    : string( ht.begin() , ht.end() )
    {
        pre_ = ht.pre_;
        tag_ = ht.tag_;
        extra_ = ht.extra_;
        post_ = ht.post_;
    }

    string print() {

        string os(pre_);
        os += "<"; 
        os += tag_;

        if (extra_.length()) {
            os += " ";
            os += extra_ ;
        }

        if ( (size()) ) {

            os += ">";
            os += std::string(begin() , end());
            os += "</";
            os += tag_;
            os += ">";
            
        } else {

            os += " />";
        }
        os += post_;

        return os;
        
    }

    std::string content()
    { return std::string( begin() , end() ); }

    HtmlTag clear_content()
    { erase( begin() , end() ); return *this; }

    HtmlTag set_tag(string t)
    { tag_ = t; return *this; }
    
    HtmlTag clear_tag() 
    { tag_ = ""; return *this; }

    HtmlTag set_pre(string p)
    { pre_ = p; return *this; }
    
    HtmlTag clear_pre() 
    { pre_ = ""; return *this; }

    HtmlTag set_post(string p) 
    { post_ = p; return *this;}
    
    HtmlTag clear_post() 
    { post_ = ""; return *this; }

    HtmlTag clear_all() 
    { erase( begin() , end() ) ; tag_="" ; pre_="" ; post_=""; return *this; }

    HtmlTag &operator<<(char* c) 
    {  append(c) ; return *this; }
    
    HtmlTag &operator<<(string s) 
    {  append(s) ; return *this;} 
    
    HtmlTag &operator<<(HtmlTag ht)    
    { append( ht.print() ); return (*this); }

    friend ostream &operator<<(ostream& os, HtmlTag ht)
        { os << ht.print(); return os; }

private:
    string tag_;
    string extra_;
    string pre_;
    string post_;
};

#endif
