#ifndef PPSSUMM_OBSERVATION_SUMMARY
#define PPSSUMM_OBSERVATION_SUMMARY

#include <vector>
#include <string>
#include <iostream>
#include <algorithm>
#include <iterator>
#include "ppssumm_typedef.h"
#include "PpssummCommon.h"
#include "Pindex.h"
#include "Eindex.h"
#include "Omindex.h"
#include "Rgsindex.h"
#include "Obsindex.h"
#include "ppssumm_algo.h"
#include "string_algo.h"
#include "key_algo.h"
#include "STime.h"
#include "Dal.h"
#include "param.h"
#include "Tracer.h"
#include "errstr.h"

class ObservationSummary
{
    public:
    ObservationSummary( const DataSet* fitsum
	,const Instsumm* instsumm
	,const Epicexposuresprocessed* epic
	,const Rgsexposuresprocessed* rgs
	,const Omexposuresprocessed* om
    ) :
	_guest(fitsum)
	,_proposal(fitsum)
	,_observation(fitsum)
	,_instrumentsummary( instsumm )
	,_exposuresummary(fitsum , epic , rgs , om)
    {
    }
    
    friend ostream& operator<<( ostream& os , const ObservationSummary& q )
    {
	// Page menu
	os  << "<div id=\"pagemenu\">\n"
	    << "<ul>\n"
	    << "<li><a href=\"#guestobserver\">Guest Observer Information</a></li>\n"
	    << "<li><a href=\"#proposal\">Proposal Target Information</a></li>\n"
	    << "<li><a href=\"#observation\">Observation Record</a></li>\n"
	    << "<li><a href=\"#instrument\">Instrument Information</a></li>\n"
	    << "<li><a href=\"#exposures\">Exposure and Configuration Information</a></li>\n"
	    << "</ul>\n"
	    << "<br clear=\"left\" />"
	    << "<hr />\n"
	    << "</div>"
	;

	// Guest observer
	os << "<div id=\"normaltable\">" << "\n";
	os << "<a name=\"guestobserver\">&nbsp;</a>" << "\n";
	os << "<h3>Guest Observer Information</h3>" << "\n";
	os << "<table>";
	os << "<tr><td><pre>SLEW OBSERVATION</pre></td></tr>";
	os << "</table>";
	os << "<hr />" << "\n";
	os << "</div>" << "\n";

	// Proposal Target Information
	os << "<div id=\"normaltable\">" << "\n";
	os << "<a name=\"proposal\">&nbsp;</a>" << "\n";
	os << "<h3>Proposal Target Information</h3>" << "\n";
	os << "<table>";
	os << q._proposal;
	os << "</table>";
	os << "<hr />" << "\n";
	os << "</div>" << "\n";

	// Observation Record
	os << "<div id=\"normaltable\">" << "\n";
	os << "<a name=\"observation\">&nbsp;</a>" << "\n";
	os << "<h3>Observation Record</h3>" << "\n";
	os << "<table>";
	os << q._observation;
	os << "</table>";
	os << "<hr />" << "\n";
	os << "</div>" << "\n";

	// Instrument Informatio
	os << "<div id=\"normaltable\">" << "\n";
	os << "<a name=\"instrument\">&nbsp;</a>" << "\n";
	os << "<h3>Instrument Information</h3>" << "\n";
	os << "<table>";
	os << q._instrumentsummary;
	os << "</table>";
	os << "<hr />" << "\n";
	os << "</div>" << "\n";

	// Exposure and Configuration Information
	os << "<div id=\"widetable\">" << "\n";
	os << "<a name=\"exposures\">&nbsp;</a>" << "\n";
	os << "<h3>Exposure and Configuration Information</h3>" << "\n";
	os << "<table>";
	os << q._exposuresummary;
	os << "</table>";
	os << "<hr />" << "\n";
	os << "</div>" << "\n";

	return os;
    }

    private:
    Guest _guest;
    Proposal _proposal;
    Observation _observation;
    Instrumentsummary _instrumentsummary;
    Exposuresummary _exposuresummary;
};
#endif
