#ifndef PPSSUMM_OBSINDEX
#define  PPSSUMM_OBSINDEX

#include <iostream>
#include <string>
#include <vector>
#include <map>
#include <algorithm>
#include <iterator>
#include <sstream>
#include <cmath>

// SAS 1
#include "Dal.h"
#include "Tracer.h"
#include "errstr.h"

// PPSSUMM Type classes
#include "ppssumm_typedef.h"
#include "ppssumm_fits.h"
#include "string_algo.h"
#include "key_algo.h"
#include "stripws.h"
#include "PpssummCommon.h"
#include "Filename_typedef.h"
#include "Filename.h"
#include "Instrument.h"
#include "Src_num.h"
#include "Band.h"
#include "Rgsorder.h"
#include "Degrees.h"
#include "Datamode.h"
#include "Pindex_typedef.h"
#include "Pindex.h"
#include "Eindex.h"
#include "Omindex.h"
#include "Rgsindex.h"


typedef std::string GuestRow;
class Guest : public std::vector<GuestRow>
{
    public:
    Guest() : std::vector<GuestRow>()
    {}
    Guest( const DataSet* fitsum ) : std::vector<GuestRow>()
    {
	std::string goget[] = {
	    "GONAME"
	    ,"GOINSTITUTE"
	    ,"GOADDRLN1"
	    ,"GOADDRLN2"
	    ,"GOADDRLN3"
	    ,"GOADDRLN4"
	    ,"GOADDRLN5"
	    ,"GOADDRLN6"
	    ,"GOEMAIL"
	    ,""
	};
    
	stringlist r = severalsingleton( fitsum , "PROPOSAL" , goget );
	for ( stringlist_ci CI = r.begin() ; CI != r.end() ; ++CI )
	{
	    std::string s =  stripws(*CI);
	    if (s.size())
		this->push_back( s );
	}
    }

    friend ostream& operator<<( ostream& os , const Guest& g )
    {
	for ( stringlist_ci CI = g.begin() ; CI != g.end() ; ++CI )
	{
	    os << "<tr><td class=\"string\">" 
		<< *CI 
		<< "</td></tr>" 
		<< "\n"
	    ;
	}
	return os;
    }
};

typedef std::string ProposalRow;
class Proposal : public std::vector<ProposalRow>
{
    public:
    Proposal() : std::vector<ProposalRow>() 
    {}
    Proposal( const DataSet* fitsum ) : std::vector<ProposalRow>() 
    {
	std::string goget[] = {
	    "TARGET"
	    ,"TARGET_RA"
	    ,"TARGET_DEC"
	    ,"PROPOSED_DURATION"
	    ,"BORESIGHT_RA"
	    ,"BORESIGHT_DEC"
	    ,"POS_ANGLE_LC"
	    ,"POS_ANGLE_UC"
	    ,"POS_ANGLE_REF"
	    ,"AO"
	    ,"SCIENCE_TYPE"
	    ,""
	};
    
	stringlist r = severalsingleton( fitsum , "PROPOSAL" , goget );
	for ( stringlist_ci CI = r.begin() ; CI != r.end() ; ++CI )
	{
	    this->push_back( *CI );
	}
    }
    
    friend ostream& operator<<( ostream& os , const Proposal& g )
    {
	os  << "<tr>" 
	    << "<td class=\"string\">" << "Target" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">SLEW</td>" 
	    << "</tr>" << "\n"
	;

	Degrees tra(g[1] , Degrees::DECIMALHOURS);
	os  << "<tr>" 
	    << "<td class=\"string\">" << "Target RA [hms]" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">n/a</td>" 
	    << "</tr>" << "\n"
	;

	Degrees tdec(g[2] , Degrees::DECIMALDEGREES );
	os  << "<tr>" 
	    << "<td class=\"string\">" << "Taget Dec [dms]" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">n/a</td>" 
	    << "</tr>" << "\n"
	;

	//os  << "<tr>" 
	//    << "<td class=\"string\">" << "Proposed Duration [s]" << "</td>" 
	//    << "<td class=\"string\">:</td>" 
	//    << "<td class=\"string\">" << g[3] << "</td>" 
	//    << "</tr>" << "\n"
	//;

	Degrees bra(g[4] , Degrees::DECIMALHOURS);
	os  << "<tr>" 
	    << "<td class=\"string\">" << "Boresight RA [hms]" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">n/a</td>" 
	    << "</tr>" << "\n"
	;

	Degrees bdec(g[5] , Degrees::DECIMALDEGREES );
	os  << "<tr>" 
	    << "<td class=\"string\">" << "Boresight Dec [dms]" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">n/a</td>" 
	    << "</tr>" << "\n"
	;

	os  << "<tr>" 
	    << "<td class=\"string\">" << "Lower Pos Ang" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">n/a</td>" 
	    << "</tr>" << "\n"
	;

	os  << "<tr>" 
	    << "<td class=\"string\">" << "Upper Pos Ang" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">n/a</td>" 
	    << "</tr>" << "\n"
	;

	os  << "<tr>" 
	    << "<td class=\"string\">" << "SC Pos Ang" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">n/a</td>" 
	    << "</tr>" << "\n"
	;

	os  << "<tr>" 
	    << "<td class=\"string\">" << "AO" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">" << g[9] << "</td>" 
	    << "</tr>" << "\n"
	;

	os  << "<tr>" 
	    << "<td class=\"string\">" << "Science Type" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">SLEW</td>" 
	    << "</tr>" << "\n"
	;
	return os;
    }
};

typedef std::string ObservationRow;
class Observation : public std::vector<ObservationRow>
{
    public:
    Observation() : std::vector<ObservationRow>() 
    {}
    Observation( const DataSet* fitsum ) : std::vector<ObservationRow>() 
    {
	std::string goget[] = {
	    "OBS_ID"
	    ,"REVOLUTION"
	    ,"OBS_START"
	    ,"OBS_STOP"
	    ,""
	};
    
	stringlist r = severalsingleton( fitsum , "OBSERVAT" , goget );
	for ( stringlist_ci CI = r.begin() ; CI != r.end() ; ++CI )
	{
	    this->push_back( *CI );
	}

	std::string odfversion = singleton( fitsum , "CONFIG" , "ODF_VERSION" );
	if (! odfversion.length() ) odfversion = "001";
	this->push_back( odfversion );
    }
    
    friend ostream& operator<<( ostream& os , const Observation& g )
    {
	os  << "<tr>" 
	    << "<td class=\"string\">" << "Observation ID" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">" << g[0] << "</td>" 
	    << "</tr>" << "\n"
	;

	os  << "<tr>" 
	    << "<td class=\"string\">" << "Revolution" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">" << g[1] << "</td>" 
	    << "</tr>" << "\n"
	;

	os  << "<tr>" 
	    << "<td class=\"string\">" << "Odf Version" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">" << g[4] << "</td>" 
	    << "</tr>" << "\n"
	;

	os  << "<tr>" 
	    << "<td class=\"string\">" << "Start time" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">" << g[2] << "</td>" 
	    << "</tr>" << "\n"
	;

	os  << "<tr>" 
	    << "<td class=\"string\">" << "Stop time" << "</td>" 
	    << "<td class=\"string\">:</td>" 
	    << "<td class=\"string\">" << g[3] << "</td>" 
	    << "</tr>" << "\n"
	;
	return os;
    }

};

class InstrumentsummaryRow
{
    public:
    InstrumentsummaryRow() {}
    InstrumentsummaryRow( const InstsummRow& r )
	: _row( r.row() )
	, _instrument( r.instrument() )
	, _active( r.active() )
	, _nexposures( r.nexposures() )
	, _priority( r.priority() )
     {}
    InstrumentsummaryRow(
        const unsigned int& r
        , const Instrument& v0
        , const Active& v1
        , const Nexposures& v2
        , const Priority& v3
    ) :
    _row(r)
    , _instrument(v0)
    , _active(v1)
    , _nexposures(v2)
    , _priority(v3)
    { }

    friend ostream& operator<<( ostream& os , const InstrumentsummaryRow r )
    {
	std::string page;

	ppssumm_internal::toString<bool8> tostring;
	os  << "<tr>"
	    << "<td class=\"string\">" 
	    << "<a href=\""
	    <<  r._instrument.pagename()
	    << "#top\">"
	    << r._instrument 
	    << "</a>"
	    << "</td>"
	    << "<td class=\"flag\">" << tostring(r._active) << "</td>"
	    << "<td class=\"string\">" << r._nexposures<< "</td>"
	    << "<td class=\"string\">" << r._priority << "</td>"
	    << "</tr>"
	;
	return os;
    }
    private:
    int _row;
    Instrument _instrument;
    Active _active;
    Nexposures _nexposures;
    Priority _priority;
};

class Instrumentsummary
: public std::vector<InstrumentsummaryRow>
{
    public:
    Instrumentsummary() : std::vector<InstrumentsummaryRow>() {}
    Instrumentsummary( const Instsumm* instsumm )
    : std::vector<InstrumentsummaryRow>()
    {
	for ( Instsumm::const_iterator CI = instsumm->begin()
	    ; CI != instsumm->end()
	    ; ++CI 
	) {
	    this->push_back( InstrumentsummaryRow( *CI ) );
	}
    }

    void title( ostream& os )
    {
	os  << "<tr>"
	    << "<th class=\"string\">Instrument</th>"
	    << "<th class=\"flag\">Active</th>"
	    << "<th class=\"string\">Exposures</th>"
	    << "<th class=\"string\">Priority</th>"
	    << "</tr>"
	;
    }
    friend ostream& operator<<( ostream& os , Instrumentsummary is)
    {
	is.title(os);
	for ( Instrumentsummary::const_iterator CI = is.begin() ; CI != is.end() ; ++CI )
	{
	    os << *CI;
	}
	return os;
    }


};

class ExposuresummaryRow
{
    public:
    ExposuresummaryRow(
	const unsigned int v0
	,const Instrument& v1
	,const Exp_id& v2
	,const Scheduled& v3
	,const Mode& v4
	,const Datamode& v5
	,const Filter& v6
	,const Position& v7
	,const Exp_duration& v8
	,const Exp_start& v10
	,const Exp_stop& v11
	,const Page& _Page
	,const Link& _Link
    ) : _row(v0)
	, _instrument(v1)
	, _exp_id(v2)
	, _scheduled(v3)
	, _mode(v4)
	, _datamode(v5)
	, _filter(v6)
	, _position(v7)
	, _exp_duration(v8)
	, _exp_start(v10)
	, _exp_stop(v11)
	, _page(_Page)
	, _link(_Link)
    {
    }

    Instrument instrument() const { return _instrument; }
    Exp_id exp_id() const { return _exp_id; }

    bool8 select( const Instrument& i , const Exp_id& e ) const
    {
	return ( _instrument == i && _exp_id == e );
    }

    bool operator<( const ExposuresummaryRow& r ) const
    {
	return (_instrument < r._instrument)
	    || ((_instrument == r._instrument) && (_exp_start < r._exp_start))
	;
    }
    
    friend ostream& operator<<( ostream& os , const ExposuresummaryRow& r )
    {
	ppssumm_internal::toString<bool8> tostring;

	os << "<tr>";

	os  << "<td class=\"string\">"  
	    << "<a href=\"" 
	    << ppssumm_internal::pagename(r._page) 
	    << "#top\">" 
	    << r._instrument 
	    << "</a>"
	    << "</td>"
	;
	
	if (r._link)
	{
	    std::string href1 = ppssumm_internal::makekey(  ppssumm_internal::pagename(r._page) , r._instrument , r._exp_id );
	    os << "<td class=\"string\">" <<  "<a href=\"" << href1 << "\">" << r._exp_id << "</a></td>" ;
	}
	else
	{
	    os << "<td class=\"string\">" << r._exp_id << "</td>";
	}

	std::ostream::fmtflags incflags = os.setf( std::ios::fixed );
	unsigned int prec = os.precision();
    
	std::string filter(r._filter);
	if ( ! filter.size() ) filter = "&nbsp;";
	
	std::string position(r._position);
	if ( ! position.size() ) position = "&nbsp;";
	
	os << "<td class=\"string\">" << tostring(r._scheduled) << "</td>"
	    << "<td class=\"string\">" << r._mode << "</td>"
	    << "<td class=\"string\">" << r._datamode << "</td>"
	    << "<td class=\"string\">" << filter << "</td>"
	    << "<td class=\"string\">" << position << "</td>"
	;
	os.precision(0);
	os  << "<td class=\"number\">" << r._exp_duration << "</td>"
	    << "<td class=\"date\">" << ppssumm_internal::truncTime(r._exp_start) << "</td>"
	    << "<td class=\"date\">" << ppssumm_internal::truncTime(r._exp_stop) << "</td>"
	    << "</tr>"
	    << "\n"
	;
	os.precision(prec);
	os.setf( incflags );
	return os;
    }

    private:
    unsigned int _row;
    Instrument _instrument;
    Exp_id _exp_id;
    Scheduled _scheduled;
    Mode _mode;
    Datamode _datamode;
    Filter _filter;
    Position _position;
    Exp_duration _exp_duration;
    Exp_start _exp_start;
    Exp_stop _exp_stop;
    Page _page;
    Link _link;
};

class Exposuresummary : public std::vector<ExposuresummaryRow>
{
    public:
    Exposuresummary() : std::vector<ExposuresummaryRow>() { }
    Exposuresummary( const DataSet* fitsum 
	,const Epicexposuresprocessed* epic
	,const Rgsexposuresprocessed* rgs
	,const Omexposuresprocessed* om
    ) : std::vector<ExposuresummaryRow>() 
    {
	unsigned int row = 0;
	std::string page;
	bool8 link;

	std::string _instname[] = { "M1" , "M2" , "PN" , "OM" , "R1" , "R2" , "" };

	const unsigned int string_num=8;
	stringlist string_cd[string_num];
	stringlist_ci string_ci[string_num];

	std::string* instname = _instname;
	while ( instname->size() )
	{
	    if ( *instname == "M1" || *instname == "M2" || *instname == "PN" ) 
		page = "epxpage";
	    else if ( *instname == "OM" ) 
		page = "omxpage";
	    else if ( *instname == "R1" || *instname == "R2" || *instname == "" ) 
		page = "rgxpage";
	    else 
		page = "";

	    unsigned int rows = 0;
	    readtable( fitsum , *instname , rows , string_cd , string_ci );
	    for(unsigned long i=0; i < rows ; i++)
	    {

		// Convert Y/N string to boolean.
		Scheduled sched  = *(string_ci[1]+i) == "Y";

		// Construct the full exposure id
		stringstream exp_id;
		exp_id << (sched ? "S" : "U");
		exp_id << *(string_ci[0]+i);


		std::string key( ppssumm_internal::makekey(  *instname , exp_id.str() ) );
		if ( *instname == "M1" || *instname == "M2" || *instname == "PN" ) 
		    link = epic->seen()->find(key) != epic->seen()->end();
		else if ( *instname == "OM" )
		    link = om->seen()->find(key) != om->seen()->end();
		else if ( *instname == "R1" || *instname == "R2" || *instname == "" )
		    link = rgs->seen()->find(key) != rgs->seen()->end();
		else
		    link = false;

		// Calculate duration from stop - start.
	
		double d = STime(*(string_ci[7]+i)) - STime(*(string_ci[6]+i));


		this->push_back( ExposuresummaryRow( row++
		    , *instname
		    , exp_id.str()
		    , sched
		    , *(string_ci[2]+i)
		    , *(string_ci[3]+i)
		    , *(string_ci[4]+i)
		    , *(string_ci[5]+i)
		    , d
		    , *(string_ci[6]+i)
		    , *(string_ci[7]+i)
		    , page
		    , link
		));
	    }
	    ++instname;
	}
	std::sort(this->begin(), this->end() , std::less<ExposuresummaryRow>() );
    }

    void readtable( const DataSet* fitsum , std::string tname , unsigned int& rows , stringlist* string_cd , stringlist_ci* string_ci )
    {
	rows = 0;
	if ( fitsum->hasBlock(tname) )
	{
	    const Table* t = fitsum->table(tname);
	    const unsigned int string_num=8;
	    //stringlist string_cd[string_num];
	    //stringlist_ci string_ci[string_num];
	    std::string string_colname[string_num] =
	    {
		    "EXPID"
		    ,"SCHEDULED"
		    ,"MODE"
		    ,"DATA_MODE"
		    ,"FILTER"
		    ,"FILTER_WHEEL"
		    ,"ACTUAL_START"
		    ,"ACTUAL_END"
	    };

	    rows = t->rows();
    
	    for(unsigned int i=0 ; i < string_num; i++)
	    {
		string_cd[i] = stringlist();
		if ( t->hasColumn( string_colname[i] ) )
		    stringColumn( t , string_colname[i] , back_inserter( string_cd[i] ) );
		else
		    string_cd[i] = stringlist(rows,"");

		string_ci[i] = string_cd[i].begin();
	    }
	}
    }
    
    void title( ostream& os ) const
    {
	os  << "<tr>"
	    << "<th class=\"string\">Inst.</th>"
	    << "<th class=\"string\">Exp&nbsp;Id</th>"
	    << "<th class=\"flag\">Sched.</th>"
	    << "<th class=\"string\">Mode</th>"
	    << "<th class=\"string\">Data Mode</th>"
	    << "<th class=\"string\">Filter</th>"
	    << "<th class=\"string\">Position</th>"
	    << "<th class=\"number\">Total Duration</th>"
	    << "<th class=\"date\">Actual Start</th>"
	    << "<th class=\"date\">Actual Stop</th>"
	    << "</tr>"
	;
    }
    friend ostream& operator<<( ostream& os , const Exposuresummary& es )
    {
	es.title(os);
	for( Exposuresummary::const_iterator CI = es.begin()
	    ; CI != es.end()
	    ; ++CI
	) {
	    os << *CI;
	}
	return os;
    }
    private: 
};

#endif
