#ifndef PPSSUMM_RGS_SUMMARY
#define PPSSUMM_RGS_SUMMARY

#include <vector>
#include <string>
#include <iostream>
#include <algorithm>
#include <iterator>
#include "ppssumm_typedef.h"
#include "PpssummCommon.h"
#include "Pindex.h"
#include "Rgsindex.h"
#include "ppssumm_algo.h"
#include "string_algo.h"
#include "key_algo.h"
#include "STime.h"
#include "Dal.h"
#include "param.h"
#include "Tracer.h"
#include "errstr.h"

class RgsSummary 
{
    public:
    RgsSummary( Pindex& pindex ) :
	_rgsmultiexposureproduct(pindex) 
	,_rgsexposuresprocessed(pindex)
	,_rgsexposurespecificproduct(pindex)
	,_rgssourcesprocessed(pindex)
	,_rgssourcespecificproduct(pindex)
    { }

    friend std::ostream &operator<<(std::ostream& os , const RgsSummary& es) {

	// Page menu
	os  << "<div id=\"pagemenu\">\n"
	    << "<ul>\n"
	    << "<li><a href=\"#mulitexposureproduct\">Multi-Exposure Products</a></li>\n"
	    << "<li><a href=\"#exposuresprocessed\">Exposures processed by PPS</a></li>\n"
	    << "<li><a href=\"#sourcesprocessed\">Sources processed by PPS</a></li>\n"
	    << "</ul>\n"
	    << "<br clear=\"left\" />"
	    << "<hr />\n"
	    << "</div>"
	;
	// Now the fun begins

	os  << "<div id=\"normaltable\">"  << "\n"
	    << "<a name=\"mulitexposureproduct\">&nbsp;</a>"
	    << "<h3>Multi-Exposure Products</h3>"
	;
	if (es._rgsmultiexposureproduct.size())
	{
	    os	<< "<table class=\"rgsmultiexposureproduct_table\">"
		<< es._rgsmultiexposureproduct
		<< "</table>"
	    ;
	}
	else
	{
	    os << "<p class=\"noprod\">No multi-exposure products were produced.</p>";
	}
	os << "<hr />";
	os << "</div>";

	// EXPOSURES

	os << "<div id=\"exposuresprocessed\">" << "\n";
	os << "<a name=\"exposuresprocessed\">&nbsp;</a>" << "\n";
	os << "<h3>Exposures processed by PPS</h3>" << "\n";
	if (es._rgsexposuresprocessed.size())
	{
	    // Exposures processed table
	    os << "<div id=\"widetable\">" << "\n";
	    os << "<table class=\"rgsexposureprocessed_table\">" << "\n";
	    os << es._rgsexposuresprocessed;
	    os << "</table>" << "\n";
	    os << "<hr />" << "\n";
	    os << "</div>" << "\n";

	    // Exposure specific products
	    os << "<div id=\"exposurespecificproducts\">" << "\n";
	    os << "<h3>Exposure Specific Products</h3>" << "\n";
	    if( es._rgsexposurespecificproduct.size() )
	    {
		for ( Rgsexposuresprocessed::const_iterator CI = es._rgsexposuresprocessed.begin()
		    ; CI != es._rgsexposuresprocessed.end()
		    ; ++CI
		){

		    // Select exposurespecificproduct for this instrument and exposure.
		    Rgsexposurespecificproduct tmp;
		    for( Rgsexposurespecificproduct::const_iterator CI2 = es._rgsexposurespecificproduct.begin()
			; CI2 != es._rgsexposurespecificproduct.end()
			; ++CI2
		    ){
			if( CI2->select( CI->instrument() , CI->exp_id() ) ) tmp.push_back( *CI2 );
		    }

		    // Repeat exposure row ahead of exposure specific product table
		    os << "<div id=\"widetable\">" << "\n";
		    os << "<a name=\"" <<  ppssumm_internal::makekey( CI->instrument() , CI->exp_id() ) << "\">&nbsp;</a>" << "\n";
		    os << "<table class=\"rgsexposurespecific_toptable\">" << "\n";
		    es._rgsexposuresprocessed.title(os);
		    os << *CI;
		    os << "</table>" << "\n";
		    os << "</div>" << "\n";

		    // Write out exposurespecificproduct table
		    if (tmp.size())
		    {
			std::sort(tmp.begin(), tmp.end() , std::less<RgsexposurespecificproductRow>() );
		    os << "<div id=\"normaltable\">" << "\n";
			os << "<table class=\"rgsexposurespecific_toptable\">" << "\n";
			os << tmp;
			os << "</table>" << "\n";
		    os << "</div>" << "\n";
		    }
		    else
		    {
			os << "<p class=\"noprod\">No exposure specific products were produced.</p>" << "\n";
		    }
		}
	    }
	    else
	    {
		os << "<p class=\"noprod\">No exposure specific products were produced.</p>" << "\n";
	    }
	    os << "<hr />" << "\n";
	    os << "</div>" << "\n";
	}
	else
	{
	    os << "<p class=\"noprod\">No RGS exposures processed by the PPS.</p>" << "\n";
	    os << "<hr />" << "\n";
	}
	os << "</div>" << "\n";


	// SOURCES

	os << "<div class=\"rgssourcesprocessed\">" << "\n";
	os << "<a name=\"sourcesprocessed\">&nbsp;</a>" << "\n";
	os << "<h3>Sources processed by PPS</h3>" << "\n";
	if (es._rgssourcesprocessed.size())
	{
	    // Sources processed table
	    os << "<div id=\"widetable\">" << "\n";
	    os << "<table class=\"rgssourceprocessed_table\">" << "\n";
	    os << es._rgssourcesprocessed;
	    os << "</table>" << "\n";
	    os << "<hr />" << "\n";
	    os << "</div>" << "\n";

	    // Source specific products
	    os << "<div class=\"rgssourcespecificproduct\">" << "\n";
	    os << "<h3>Source Specific Products</h3>" << "\n";
	    if( es._rgssourcespecificproduct.size() )
	    {	
		for ( Rgssourcesprocessed::const_iterator CI = es._rgssourcesprocessed.begin()
		    ; CI != es._rgssourcesprocessed.end()
		    ; ++CI
		){
		    // Select source specific product for this instrument and source.
		    Rgssourcespecificproduct tmp;
		    for( Rgssourcespecificproduct::const_iterator CI2 = es._rgssourcespecificproduct.begin()
			; CI2 != es._rgssourcespecificproduct.end()
			; ++CI2
		    ){
			if( CI2->select( CI->instrument() , CI->exp_id(), CI->src_num() ) ) tmp.push_back( *CI2 );
		    }

		    // Repeat source row ahead of source specific product table
		    os  << "<div id=\"widetable\">" << "\n"
			<< "<a name=\"" <<  ppssumm_internal::makekey( CI->instrument() , CI->exp_id() , CI->src_num() ) << "\">&nbsp;</a>" << "\n"
			<< "<table class=\"rgssourcespecific_toptable\">" << "\n"
		    ;
		    es._rgssourcesprocessed.title(os);
		    os	<< *CI
			<< "</table>" << "\n"
			<< "</div>" << "\n"
		    ;

		    // Write out sourcespecificproduct table
		    if (tmp.size())
		    {
			std::sort(tmp.begin(), tmp.end() , std::less<RgssourcespecificproductRow>() );
			os  << "<div id=\"normaltable\">" << "\n"
			    << "<table class=\"rgssourcespecific_toptable\">" << "\n"
			    << tmp
			    << "</table>" << "\n"
			    << "</div>"
			;
		    }
		    else
		    {
			os << "<p class=\"noprod\">No source specific products were produced.</p>" << "\n"
			;
		    }
		}
	    }
	    else
	    {
		os  << "<p class=\"noprod\">No source specific products were produced.</p>" << "\n"
		;
	    }
	    os  << "<hr />" << "\n"
		<< "</div>" << "\n"
	    ;
	}
	else
	{
	    os  << "<p class=\"noprod\">No RGS sources processed by the PPS.</p>" << "\n"
	    ;
	    os  << "<hr />" << "\n";
	}
	os  << "</div>" << "\n";

        return os;
    }
   
    const Rgsexposuresprocessed* rgsexposuresprocessed() const { return &_rgsexposuresprocessed; }
 
    private:
    Rgsmultiexposureproduct _rgsmultiexposureproduct;
    Rgsexposuresprocessed _rgsexposuresprocessed;
    Rgsexposurespecificproduct _rgsexposurespecificproduct;
    Rgssourcesprocessed _rgssourcesprocessed;
    Rgssourcespecificproduct _rgssourcespecificproduct;

};

#endif
