/*
 ppsslewsumm

   Authors:  Duncan John Fyfe - SSC/LUX
             Duncan Law-Green - SSC/LUX
 
   Description: Summerise Results of PCMS Processing
 
   Comments: 
 
  $Id: module.cc,v 1.5 2000/10/02 08:30:13 gvacanti Exp $
*/


#include <fstream>
#include <string>
#include <iostream>

#include "Pindex.h"
#include "getSumFile.h"
#include "ObsSummary.h"

#include "PpssummCommon.h"
#include "ObservationSummary.h"
#include "EpicSummary.h"
#include "ProductSummary.h"
#include "OmSummary.h"
#include "RgsSummary.h"
#include "CatSummary.h"
#include "HtmlPage.h"

#include "Dal.h"
#include "ppsslewsumm.h"
#include "param.h"
#include "Tracer.h"
#include "errstr.h"

void ppsslewsumm ()
{
PROC
    errstr << "Begin ppsslewsumm" << Sas::message(Msg::App, Msg::Verbose);

    /*
     * Read observation summary file , convert to a temporary fits file.
     * Hopefully this will be split into a separate task in the devtrack
     */

    DataSet* fitsum;
    std::string optname(stringParameter("fitsum"));
    if ( optname.empty() ) 
    {
        optname = tempnam(0,0);
    }
    fitsum = dataSetServer->open( optname.c_str() , DataSet::Create );

    //std::string fitsum = (optname != 0)
    //    ? optname
    //    : tempnam(0,0)
    //;

    //DataSet* fitsum = dataSetServer->open( fitsum , DataSet::Temp ); 
    //DataSet* fitsum = dataSetServer->open( optname.c_str() , DataSet::Temp ); 

    std::string inputSummaryFile = getSumFile();
    ObsSummary( inputSummaryFile , fitsum );
    dataSetServer->close(fitsum);
    fitsum = dataSetServer->open( optname.c_str() , DataSet::Read );
    
    /*
     * Open product index 
     */
    std::string pindex_file( stringParameter("pindex") );

    Pindex pindex( pindex_file );

    /* initialize common data */
    PpssummCommon* pc = new PpssummCommon( fitsum , pindex_file );


    /* Each page must be created before output for cross-page links to 
     * be built
     */

    std::string catpage_fname = stringParameter("catpage");
    CatPage catpage( catpage_fname.c_str() );
    EpicSummary epxpage( fitsum , pindex );
    OmSummary omxpage( pindex  );
    RgsSummary rgxpage( pindex );
    ProductSummary ppspage( pindex , epxpage.epicexposuresprocessed() , rgxpage.rgsexposuresprocessed() , omxpage.omexposuresprocessed() );
    ObservationSummary obspage( fitsum , pindex.instsumm() ,  epxpage.epicexposuresprocessed() , rgxpage.rgsexposuresprocessed() , omxpage.omexposuresprocessed() );

    HtmlPage html( pc );    

    std::string fname; 
    std::ofstream out;
    std::string page("");
      
    errstr << "Writing obspage" << Sas::message(Msg::App, Msg::Sparse);
    page = "obspage";
    fname = stringParameter(page.c_str());
    out.open(fname.c_str(), std::ios::out);
    if (out)
    {
	html.page_id(page);
	html.start( out );
	out << obspage;
	html.end( out );
	out.close();
    }
    
    errstr << "Writing ppspage" << Sas::message(Msg::App, Msg::Sparse);
    page = "ppspage";
    fname = stringParameter(page);
    out.open(fname.c_str(), std::ios::out);
    if (out) 
    {
	html.page_id(page);
	html.start( out );
	out << ppspage;
	html.end( out );
	out.close();
    }
    
    errstr << "Writing epxpage" << Sas::message(Msg::App, Msg::Sparse);
    page = "epxpage";
    fname = stringParameter(page);
    out.open(fname.c_str(), std::ios::out);
    if (out) 
    {
	html.page_id(page);
	html.start( out );
	out << epxpage;
	html.end( out );
	out.close();
    }
    
    errstr << "Writing omxpage" << Sas::message(Msg::App, Msg::Sparse);
    page = "omxpage";
    fname = stringParameter(page);
    out.open(fname.c_str(), std::ios::out);
    if (out) 
    {
    	html.page_id(page);
    	html.start( out );
    	out << omxpage;
    	html.end( out );
    	out.close();
    }

    errstr << "Writing rgxpage" << Sas::message(Msg::App, Msg::Sparse);
    page = "rgxpage";
    fname = stringParameter(page);
    out.open(fname.c_str(), std::ios::out);
    if (out) 
    {
    	html.page_id(page);
    	html.start( out );
    	out << rgxpage;
    	html.end( out );
    	out.close();
        }
    
    /* Create dummy cat page if one does not exist */
    if ( ! catSummaryExists(catpage_fname.c_str()) )
    {
    errstr << "Writing catpage" << Sas::message(Msg::App, Msg::Sparse);
	page="catpage";
	out.open(catpage_fname.c_str(), std::ios::out);
	if (out)
	{
	    html.page_id(page);
	    html.start( out );
	    out << catpage;
	    html.end( out );
	    out.close();
	}
    }
    
    delete pc;
    dataSetServer->close(fitsum);

    errstr << "End ppsslewsumm" <<  Sas::message(Msg::App, Msg::Verbose);
ENDPROC
}
